#!/bin/sh

# This shell script supports fstab operations for archivepi and networkstoragepi.

# Process fstab symlinks
original_file_path="/etc/fstab"
if [ -L "$original_file_path" ]; then
  real_file_path=$(readlink -f "$original_file_path")
  if [ -z "$real_file_path" ]; then
    real_file_path=$(readlink "$original_file_path")
  fi
else
  real_file_path="$original_file_path"
fi
temp_file_path_ext=".ev"
temp_file_path="$real_file_path$temp_file_path_ext"
old_file_path_ext=".old"
old_file_path="$real_file_path$old_file_path_ext"

read_devices() {
  while IFS=' ' read -r device dir type options dump fsck
  do
    case "$device" in
        # Skip any line beginning with hashtag denoting comment.
        "#"*) continue;;
        *) echo "$device $dir $type $options $dump $fsck";;
    esac
  done < "$original_file_path"
}
  
copy_comments() {
  while IFS= read -r comments
    do
      case "$comments" in
      "#"*) echo "$comments" >> "$temp_file_path";;
      *) continue;;
      esac
  done < "$original_file_path"
}

create() {
 true > "$temp_file_path"
}

set_permission() {
  chmod 644 "$temp_file_path"
}

rename() {
  temp=$1
  original=$2
  cp "$original" "$old_file_path"
  mv "$temp" "$original"
}

validate_parameters() {
  for v in "$@"
  do
    w="${v%%"[[:space:]\\]*"}"
    if [ "$v" != "$w" ]; then 
      echo "Illegal char detected while attempting to add entry!"
      return 1
    fi
  done
  return 0
}

add_entry() {
  validate_parameters "$@"
  validate_parameters_code=$?
  
  if [ "$validate_parameters_code" -eq "1" ]; then
    exit 2
  fi
  
  path=$1
  device=$2
  dir=$3
  type=$4
  options=$5
  dump=$6
  fsck=$7

  echo "$device $dir $type $options $dump $fsck" >> "$path"
  return 0
}

##### Main
if [ "$1" = "read" ]; then
  read_devices
elif [ "$1" = "create" ]; then
  create
  copy_comments
  set_permission
elif [ "$1" = "rename" ]; then
  shift
  rename "$@"
elif [ "$1" = "addentry" ]; then
  shift
  add_entry "$@"
else
  echo "Unknown option $1. Use read, create, rename, or addentry."
  exit 1
fi

exit 0
