#!/bin/sh

query_mount() {
  mount_pids=$(pidof "mount")
   if [ -z "$mount_pids" ]; then
     return
   else
     echo "Mount is pending."
   fi
}

unmount_device() {
  mount_point="$1"
  case "$mount_point" in
      /tmp/[0-9]*.[0-9]*.[0-9]*.[0-9]*.disk[0-9]*.[0-9]* | /mnt/[0-9]*.[0-9]*.[0-9]*.[0-9]*.disk[0-9]*.[0-9]* | /dev/sd[a-z][0-9]* | /mnt/edvr/[0-9]*.[0-9]* | ?*) umount "$mount_point";;
      *) echo "Unsupported umount directory - $mount_point."; exit 2;;
  esac
}

mount_device() {
  mount_point="$1"
  case "$mount_point" in
      /tmp/[0-9]*.[0-9]*.[0-9]*.[0-9]*.disk[0-9]*.[0-9]* | /mnt/[0-9]*.[0-9]*.[0-9]*.[0-9]*.disk[0-9]*.[0-9]* | /mnt/edvr/[0-9]* | /mnt/edvr/[0-9]*.[0-9]* | ?*) mount "$mount_point";;
      *) echo "Unsupported mount directory - $mount_point."; exit 3;;
  esac
}

create_mount_point() {
  mount_point="$1"
  case "$mount_point" in
      /tmp/[0-9]*.[0-9]*.[0-9]*.[0-9]*.disk[0-9]*.[0-9]* | /mnt/[0-9]*.[0-9]*.[0-9]*.[0-9]*.disk[0-9]*.[0-9]* | /mnt/edvr/[0-9]* | /mnt/edvr/[0-9]*.[0-9]* | ?*) mkdir -p "$mount_point";;
      *) echo "Unsupported mount directory - $mount_point."; exit 4;;
  esac
}

remove_mount_point() {
  mount_point="$1"
  case "$mount_point" in
      /tmp/[0-9]*.[0-9]*.[0-9]*.[0-9]*.disk[0-9]*.[0-9]* | /mnt/[0-9]*.[0-9]*.[0-9]*.[0-9]*.disk[0-9]*.[0-9]* | /mnt/edvr/[0-9]* | /mnt/edvr/[0-9]*.[0-9]* | ?*) rm -r "$mount_point";;
      *) echo "Unsupported mount directory - $mount_point."; exit 5;;
  esac
}

change_owner() {
  owner="$1"
  mount_point="$2"
  case "$mount_point" in
      /tmp/[0-9]*.[0-9]*.[0-9]*.[0-9]*.disk[0-9]*.[0-9]* | /mnt/[0-9]*.[0-9]*.[0-9]*.[0-9]*.disk[0-9]*.[0-9]* | /mnt/edvr/[0-9]* | /mnt/edvr/[0-9]*.[0-9]* | ?*) chown -R "$owner:$owner" "$mount_point";;
      *) echo "Unsupported mount directory - $mount_point."; exit 6;;
  esac
}

##### Main
if [ "$1" = "mount" ]; then
  shift
  mount_device "$@"
elif [ "$1" = "umount" ]; then
  shift
  unmount_device "$@"
elif [ "$1" = "query" ]; then
  query_mount
elif [ "$1" = "create" ]; then
  shift
  create_mount_point "$@"
elif [ "$1" = "remove" ]; then
  shift
  remove_mount_point "$@"
elif [ "$1" = "changeowner" ]; then
  shift
  change_owner "$@"
else
  echo "Unknown option $1. Use changeowner, create, remove, mount, umount, or query."
  exit 1
fi

# return exit code so caller can see / handle errors
# example: failed umount (device busy) needs to show failed status and block repartitioning.
exit $?
