#!/bin/sh

#
#

CURL=./curl
curlopts="--anyauth -s -S --connect-timeout 10 --max-time 10"

NIC=
MAC=
IFINDEX=
CONNECTED=
SPEED=
DHCP=
IP=
MASK=
GW=
DNS=

USERNAME=root
PASSWORD=pass

# function to get username and password
get_creds()
{
    # curl needs to know the username/password of the camera so find that out the hard way
    line=`grep password ./axispi.xml`
    
    rc=$?
    if [ $rc -ne 0 ]
    then
        echo "Error: failed to get credentials with error $rc"
        exit $rc
    fi

    #
    # this is kind of convoluted since ash doesn't do all the string matching of bash
    for word in $line
    do
    case $word in
      password*) pass=`echo $word | cut -d \= -f2` ;;
  
      username*) user=`echo $word | cut -d \= -f2` ;;
    esac
    done
    
    ##
    # now strip leading and trailing " from username/password
    #
    user=${user#\"}
    USERNAME=${user%\"}
    pass=${pass#\"}
    PASSWORD=${pass%\"}
}

# function to print get network settings
print_network_config()
{
    USER=$1
    PWD=$2

    URL="http://127.0.0.1/axis-cgi/param.cgi?action=list&group=network.bootproto"
    URL="$URL,network.eth0.ipaddress,network.eth0.subnetmask,network.routing.defaultrouter,network.resolver.nameserver1"

    output=`$CURL $curlopts -u $USER:$PWD "$URL" 2>&1`

    rc=$?
    if [ $rc -ne 0 ]
    then
        echo "Error: failed to get network settings with error $rc"
        echo "$output"
        exit $rc
    fi
    
    for l in $output
    do
        case $l in
            network.bootproto=*) DHCP=`echo $l | cut -d \= -f2 | tr -d "\r"` ;;
            network.eth0.ipaddress=*) IP=`echo $l | cut -d \= -f2 | tr -d "\r"` ;;
            network.routing.defaultrouter=*) GW=`echo $l | cut -d \= -f2 | tr -d "\r"` ;;
            network.eth0.subnetmask=*) MASK=`echo $l | cut -d \= -f2 | tr -d "\r"` ;;
            network.resolver.nameserver1=*) DNS=`echo $l | cut -d \= -f2 | tr -d "\r"` ;;
        esac
    done
    
    if [ -z "$IP" ]
    then
        echo "Error: Failed to get settings. $output"
        exit 1
    fi

    MAC=`cat /sys/class/net/eth0/address`

    IFINDEX=`cat /sys/class/net/eth0/ifindex`
    if [ -z "$IFINDEX" ]; then
        IFINDEX=2
    fi

    CONNECTED=`cat /sys/class/net/eth0/carrier`

    SPEED=`cat /sys/class/net/eth0/speed`
    if [ -z "$SPEED" ]; then
        SPEED=100
    fi

    if [ $DHCP = "dhcp" ]
    then
        DHCP=1
    else
        DHCP=0
    fi
    
    echo "reboot=0"
    echo "nic=eth0"
    echo "mac=$MAC"
    echo "index=$IFINDEX"
    echo "connected=$CONNECTED"
    echo "linkspeed=$SPEED"
    echo "dhcp=$DHCP"
    echo "ip=$IP"
    echo "netmask=$MASK"
    echo "gateway=$GW"
    echo "dns=$DNS"
}
# # ---- end helper functions above ---- # #

if [ $# -lt 1 ]
then
    echo "Error: Must specify get or set parameter"
    exit 1
fi

get_creds
if [ $1 = "get" ]
then
    print_network_config $USERNAME $PASSWORD
elif [ $1 = "set" ]
then
    if [ $# -lt 2 ]
    then
        echo "Error: Must specify network interface name"
        exit 1
    fi
    
    NIC=$2
    shift 2 #now $1 is the first parameter after interface name

    do_update=0
    while :;
    do
        case $1 in
            -ip)
                if [ -n "$2" ]; then
                    IP=$2
                    shift 2
                    do_update=1
                    continue
                else
                    echo "Error: missing argument for -ip"
                    exit 1
                fi
                ;;
            -mask)
                if [ -n "$2" ]; then
                    MASK=$2
                    shift 2
                    do_update=1
                    continue
                else
                    echo "Error: missing argument for -mask"
                    exit 1
                fi
                ;;
            -gw)
                if [ -n "$2" ]; then
                    GW=$2
                    shift 2
                    do_update=1
                    continue
                else
                    echo "Error: missing argument for -gw"
                    exit 1
                fi
                ;;
            -dns)
                if [ -n "$2" ]; then
                    DNS=$2
                    shift 2
                    do_update=1
                    continue
                else
                    echo "Error: missing argument for -dns"
                    exit 1
                fi
                ;;
            -dhcp)
                if [ -n "$2" ]; then
                    DHCP=$2
                    shift 2
                    do_update=1
                    continue
                else
                    echo "Error: missing argument for -dhcp"
                    exit 1
                fi
                ;;
            -?*)
                echo "Error: unknown option $1"
                exit 1
                ;;
            *) break
        esac
        shift
    done

    if [ $do_update -eq 0 ]
    then
        echo "Error: no valid update parameters found"
        exit 1
    fi

    #build url for curl
    URL="http://127.0.0.1/axis-cgi/param.cgi?action=update"
    if [ -n "$DHCP" ]; then
        if [ "$DHCP" = "1" ]; then
            URL="$URL&network.bootproto=dhcp"
        else
            URL="$URL&network.bootproto=none"
        fi
    fi
    if [ -n "$IP" ]; then
        URL="$URL&network.ipaddress=$IP"
    fi
    if [ -n "$MASK" ]; then
        URL="$URL&network.subnetmask=$MASK"
    fi
    if [ -n "$GW" ]; then
        URL="$URL&network.defaultrouter=$GW"
    fi
    if [ -n "$DNS" ]; then
        URL="$URL&network.dnsserver1=$DNS"
    fi
    
    output=`$CURL $curlopts -u $USERNAME:$PASSWORD "$URL" 2>&1`
    
    if [ $? -ne 0 ]
    then
        echo "Error: curl command to update settings using ($URL) failed with error $?"
        exit $?
    fi
    
    # check that output starts with OK., error otherwise
    case $output in
        *OK*) ;;
        *)
            echo "Error: Failed to update network settings via URL ($URL)."
            echo "$output"
            exit 1
            ;;
    esac

    print_network_config $USERNAME $PASSWORD    
fi

