#!/bin/sh -e

# dash on ubuntu <= 12.04 does not support [^...]
# mozart does not support [[:space:]]
# artpec-4 does not support grep -m1

parse_dst()
{
  dst=${1#<*>}
  dst=${dst#*[0-9]}
  dst=${dst#[0-9]}
  dst=${dst#:[0-9][0-9]}
}

parse_posix()
{
  local dst p1 p2
  parse_dst "$1"
  p1=${1%$dst}
  p2=${dst%%,*}

  start_end=${1#$p1}
  start_end=${start_end#$p2}

  off1=${p1#<*>}
  off1=${off1##*[^0-9:-]}
  off2=${p2#<*>}
  off2=${off2##*[^0-9:-]}
}

format_name()
{
  local name="$1"
  local posix="$2"
  local off1 off2 start_end
  local sign='-'
  local oh
  local om=0
  local dst

  parse_posix "$posix"
  oh=${off1%:*}
  if [ "$oh" != "$off1" ]; then
    om=${off1#*:}
  fi
  if [ "$oh" != "${oh#-}" ]; then
    sign='+'
    oh=${oh#-}
  fi

  name=${name#(*) }
  if [ "$os" = "aarch64" ]; then
    [ -z "${name##* (+DST)}" ] && dst=' (+DST)' || dst=''
    name=${name% (+DST)}
  else
    name=${name% (+DST)}
    [ -n "$start_end" ] && dst=' (+DST)' || dst=''
  fi
  printf "(UTC%c%02d:%02d) %s%s\n" "$sign" "$oh" "$om" "$name" "$dst"
}

add_timezone_to_server()
{
  local name="$1"
  local iana="$2"
  local posix="$3"
  local cnamefl cianafl cgmtfl ciposixfl
  if [ 0 -ne $cname ]; then
    cnamefl="$cachedname"
  fi
  if [ 0 -ne $ciana ]; then
    cianafl="$cachediana"
  fi
  if [ 0 -ne $cposix ]; then
    cposixfl="$cachedposix"
  fi
  if [ 0 -ne $cgmt ]; then
    cgmtfl="$cachedgmt"
  fi
  echo "$name|$cnamefl$cianafl$cgmtfl$cposixfl$iana|$posix"
}

build_posix()
{
  local line="$1"
  local name="$2"
  local iana="$3"
  local posix="$4"
  local zonesposix="$5"
  local tempiana

  #if we get here it means the config file has neither matching iana
  #no matching posix
  tempiana=${line%% *}
  tempiana=${tempiana#/usr/share/zoneinfo/}
  #if info in tz-list-uniq.conf matches the info in zones.txt.orig
  #then use the iana name ASIS - cache it for tzset
  if [ -z "${tempiana##Etc/GMT*}" ]; then
    #we do not need to build an argument for tzset, just use iana ASIS
    #also we do not need to buid server's posix value because it matches
    #the one we read from $zonesfile.orig, so posix=$posix in this case
    #except the cases for an empty input iana
    cgmt=1
    tempiana=${tempiana#Etc/}
  fi

  local tzconfposix=${line##* }
  local angleinput=0
  if [ -z "${zonesposix##<*}" ]; then
    angleinput=1
  fi

  #build timezoneinfo for the input patterns like TZName||posix
  if [ -z "$iana" ]; then
    if [ 0 -eq $angleinput  ]; then
      #the case for timezones like
      #Israel||IST-2
      #we need to build posix since the input posix from zones.txt
      #would not match the one in /etc/localtime
      if [ 1 -eq $cgmt ]; then
        ciana=1
        local sign=${off1%%[0-9]*}
        off1=${off1#-}
        local prefix
        if [ 1 -eq ${#off1} ]; then
          prefix='0'
        fi
        if [ "$sign" = "-" ]; then
          prefix="+$prefix"
        else
          sign='+'
          prefix="-$prefix"
        fi
        #use posix built from $zonesfile.orig
        posix="<$prefix$off1>$sign$off1"
      else
        #use posix from tz-list-uniq.conf for tzset
        posix=${line##* }
        cposix=1
      fi
    else
      if [ 1 -eq $angleinput ]; then
        if [ 0 -eq $cgmt ]; then
          #use posix from zones.txt.orig for /etc/localtime
          #and query tz-list-uniq.conf by iana from tz-list-uniq.conf
          #to use tzset with tz-list-uniq.conf  posix
          posix="$zonesposix"
          ciana=1
        else
          #use posix from tz-list-uniq.conf for tzset
          posix=${line##* }
          cposix=1
        fi
      fi
    fi
  #non-empty iana
  else
    if [ 0 -eq $cgmt ]; then
      if [ 0 -ne $angleinput ]; then
        #input iana is not empty and input posix has "<". We will have to use
        #the config file posix to guarantee /opt/ad/bin/tzset success on setting
        #time zone, but we have to match /etc/localtime against the input posix
        #so cahche the iana flag
        ciana=1
      else
        if [ 0 -eq $angleinput  ]; then
          #input iana is not empty and the matching posix from tz-list-uniq.conf is not <GMT
          #and the input posix is not angled like in Chatham Islands (+DST)|Pacific/Chatham|<+1245>-12:45<+1345>,M9.5.0/2:45,M4.1.0/3:45
          #use this matching posix to guarantee /opt/ad/bin/tzset success
          posix=${line##* }
          cposix=1
        else
          #use posix from zones.txt.orig for /etc/localtime
          #and query tz-list-uniq.conf by iana from tz-list-uniq.conf
          #to use tzset with tz-list-uniq.conf  posix
          posix="$zonesposix"
          ciana=1
        fi
      fi
    fi
  fi

  iana="$tempiana"
  add_timezone_to_server "$name" "$iana" "$posix"
}

build_posix_for_iana()
{
  local line="$1"
  local name="$2"
  local iana="$3"
  local posix="$4"

  local tzconfposix=${line##* }
  if [ -z "${iana##Etc/GMT*}" ]; then
    #we do not need to build an argument for tzset, just use iana ASIS
    #also we do not need to buid server's posix value because it matches
    #the one we read from $zonesfile.orig, so posix=$posix in this case
    iana=${iana#Etc/}
    cgmt=1
  else
    local angleinput=0
    if [ -z "${posix##<*}" ]; then
      angleinput=1
    fi
    if [ 0 -eq $angleinput  ]; then
     #matching posix from tz-list-uniq.conf is neither <GMT nor <[9]+
     #use this matching posix to guarantee /opt/ad/bin/tzset success
      posix=${line##* }
      cposix=1
    else
      #use posix from zones.txt.orig for /etc/localtime
      #and query tz-list-uniq.conf by iana from tz-list-uniq.conf
      #to use tzset with tz-list-uniq.conf  posix
      ciana=1
     fi
  fi
  add_timezone_to_server "$name" "$iana" "$posix"
}

build_i610()
{
  local name iana posix line dst
  local off1 off2 start_end
  local any="\([A-Z]\+\|<[^>]*>\)"

  while IFS='|' read name iana posix; do
    if ! line=$(grep -m1 "|$posix$" /etc/sysconfig/tzlist.conf); then
      parse_posix "$posix"
      dst="$off2$start_end"
      if [ -n "$dst" ]; then
        dst="$any$dst"
      fi
      line=$(grep -m1 "|$any$off1$dst$" /etc/sysconfig/tzlist.conf) || true
    fi
    if [ -n "$line" ]; then
      posix=${line##*|}
      iana=${line#*|}
      iana=${iana#*|}
      iana=${iana%%|*}
      if [ "$iana" != "${iana#Etc/}" ]; then
        posix='<'${iana#Etc/}'>'${posix#GMT}
      fi
      echo "$name|$iana|$posix"
    fi
  done < "$zonesfile.orig" > "$zonesfile"
}

build_i825()
{
  local name iana posix line dst
  local off1 off2 start_end
  local any="\([A-Z]\+\|<[^>]*>\)"

  while IFS='|' read name iana posix; do
    if ! line=$(grep -m1 " $posix$" /opt/ad/etc/conf/tz-list-uniq.conf); then
      parse_posix "$posix"
      dst="$off2$start_end"
      if [ -n "$dst" ]; then
        dst="$any$dst"
      fi
      line=$(grep -m1 " $any$off1$dst$" /opt/ad/etc/conf/tz-list-uniq.conf) || true
    fi
    if [ -n "$line" ]; then
      posix=${line##* }
      iana=${line%% *}
      iana=${iana#/usr/share/zoneinfo/}
      echo "$name|$iana|$posix"
    fi
  done < "$zonesfile.orig" > "$zonesfile"
}

build_aarch64_zones()
{
  local name iana posix line foundiana tempiana dst
  local off1 off2 start_end
  local any="\([A-Z]\+\|<[^>]*>\)"

  while IFS='|' read name iana posix; do
    cname=1
    ciana=0
    cgmt=0
    cposix=0
    local zonesposix="$posix"
    #check whether a non-empty iana timezone is in config file
    if { [ -n "$iana" ]; } && line=$(grep -m1 "$iana " /opt/ad/etc/conf/tz-list-uniq.conf); then
      cname=0
      if [ -z "${line##*$posix*}" ]; then
        #use posix from the config file
        cposix=1
        add_timezone_to_server "$name" "$iana" "$posix"
      else
        build_posix_for_iana "$line" "$name" "$iana" "$posix"
      fi
    else
      #find a matching posix
      #iana does not match so use the name from the original zones file for /etc/timezone
      if ! line=$(grep -m1 " $posix$" /opt/ad/etc/conf/tz-list-uniq.conf); then
        parse_posix "$posix"
        dst="$off2$start_end"
        if [ -n "$dst" ]; then
          dst="$any$dst"
        fi
        line=$(grep -m1 " $any$off1$dst$" /opt/ad/etc/conf/tz-list-uniq.conf) || true
        if [ -n "$line" ]; then
          cname=0
          build_posix "$line" "$name" "$iana" "$posix" "$zonesposix"
        fi
      else
        #posix matches the one in tz-list-uniq.conf so use both: iana and posix from the conf file
        #but cache the name for /ect/timezone
        #no need to check for angles because the original zones file does not use 'GMT' in posix while
        # the conf file does
        cname=0
        cposix=1
        iana=${line%% *}
        iana=${iana#/usr/share/zoneinfo/}
        add_timezone_to_server "$name" "$iana" "$posix"
      fi
    fi
  done < "$zonesfile.orig" > "$zonesfile"
}


build_i825_aarch64()
{
  local kernel_release
  kernel_release=$(uname -r)
  if [ -n $kernel_release ]; then
    if case $kernel_release in 2*) true;; *) false;; esac; then
      os="i825"
    else
      os="aarch64"
    fi
    if [ ! -e "$zonesfile" ]; then
      if [ "i825" = "$os" ]; then
        build_i825
      else
        build_aarch64_zones
      fi
    fi
  else
    echo 'unknown platform.'
    exit 1
  fi
}

list_timezones()
{
  local name iana posix
  while IFS='|' read name iana posix; do
    format_name "$name" "$posix"
  done < "$zonesfile"
}

write_tzfile()
{
  local posix=$1
  local z3='\0\0\0'
  local z7=$z3$z3'\0'
  local z18=$z7$z7$z3'\0'
  local z25=$z18$z7
  local z29=$z25$z3'\0'
  printf "TZif2$z18\2$z3\2$z7\1$z3\2$z25"
  printf "TZif2$z18\2$z3\2$z7\1$z3\2$z29"
  printf "\n$posix\n"
}

find_timezone()
{
  local line

  name="$1"
  getfl=$2
  name=${name#(*) }
  local cachedsearch
  if [ "$os" = "aarch64" ]; then
    cachedsearch="[$cachedname]*[$cachediana]*[$cachedgmt]*[$cachedposix]*$name"
  else
    cachedsearch="$name"
  fi
  if line=$(grep -e "^$name|" -e "|$cachedsearch|" "$zonesfile" | head -n1); then
    name=${line%%|*}
    iana=${line#*|}
    iana=${iana%|*}
  if [[ "$os" = "aarch64" && 0 -ne $getfl ]]; then
    iana=$(get_pureiana $iana)
  fi
    posix=${line##*|}
  elif [ -f "/usr/share/zoneinfo/$name" ]; then
    iana=$name
    parse_tzfile "/usr/share/zoneinfo/$iana"
    line=$(grep "|$posix$" "$zonesfile" || true)
    name=${line%%|*}
  else
    name=''
    iana=''
    posix=''
  fi
}

get_pureiana()
{
  local pureiana="$1"
  pureiana=${pureiana#$cachedname*}
  pureiana=${pureiana#$cachediana*}
  pureiana=${pureiana#$cachedgmt*}
  pureiana=${pureiana#$cachedposix*}
  echo "$pureiana"
}

parse_tzfile()
{
  local tzfile="$1"
  local has_newline=$(( $(tail -c1 "$tzfile" 2> /dev/null | wc -l) ))
  if [ "$has_newline" = "1" ]; then
    posix=$(tail -n1 "$tzfile")
  else
    local abbrev=$(TZ="$tzfile" date +'%Z')
    local offset=$(TZ="$tzfile" date +'%z')
    local sign=${offset%[0-9]*}
    [ "$sign" = "-" ] && sign='' || sign='-'
    local hour=${offset#[+-]}
    hour=${hour%[0-9][0-9]}
    hour=${hour#0}
    local min=${offset#[+-]}
    min=${min#[0-9][0-9]}
    offset="$sign$hour"
    if [ "$min" != '00' ]; then
      offset="$offset:$min"
    elif [ "$hour" = '0' -a "$sign" = '-' ]; then
      offset='0'
    fi
    posix="$abbrev$offset"
  fi
}

get_timezone()
{
  local name iana posix expected

  if [ -e "$namefile" ]; then
    name=$(cat "$namefile")
    find_timezone "$name" 1
  fi

  if [ "$os" != 'vivotek' ]; then
    expected=$posix
    parse_tzfile /etc/localtime
    if [ "$posix" != "$expected" ]; then
      name=$(grep "|$posix$" "$zonesfile" || true)
      name=${name%%|*}
      #if [ -z "$name" ]; then
        #name=$(grep "|$posix$" "$zonesfile.orig" || true)
        #name=${name%%|*}
      #fi
    fi
  fi

  if [ -z "$posix" ]; then
    name='UTC'
    posix='UTC0'
  fi
  if [ -z "$name" ]; then
    name='Custom'
  fi
  format_name "$name" "$posix"
  echo "$posix"
}

set_timezone()
{
  local name iana ianacache posix desired_posix dst

  find_timezone "$1" 0
  if [ -z "$name" ]; then
    echo 'timezone not found.'
    exit 1
  fi

  local desired_posix="$posix"

  case "$os" in
  axis)
    parse_dst "$posix"
    [ -n "$dst" ] && dst='yes' || dst='no'
    parhandclient set time.posixtimezone "$posix"
    parhandclient set time.dst.enabled "$dst"
    # axis will accept any posix string
    # wait for new setting
    parhandclient get time.posixtimezone
    ;;
  i610)
    local prev=$(paramclient -get -raw system.datetime.tz.name)
    if [ "$posix" != "${posix#<}" ]; then
      posix=GMT${posix#<*>}
    fi
    paramclient -set "system.datetime.tz.name=$posix"
    if /etc/init.d/timezone start; then
      paramclient -set "system.datetime.tz.posixrule=$posix"
    else
      paramclient -set "system.datetime.tz.name=$prev"
      exit 1
    fi
    ;;
  i825)
    if [ "$posix" != "${posix#<}" ]; then
      posix=${posix#<}
      posix=${posix%%>*}
    fi
    /opt/ad/bin/tzset "$posix"
    ;;
  aarch64)
    cname=0
    case "$iana" in
      *"$cachedgmt"*)
        iana=$(get_pureiana $iana)
        /opt/ad/bin/tzset "$iana"
        ;;
      *"$cachediana"*)
        cname=1
        iana=$(get_pureiana $iana)
        if line=$(grep -m1 "$iana " /opt/ad/etc/conf/tz-list-uniq.conf); then
          if [ -n "$line" ]; then
            posix=${line##* }
            /opt/ad/bin/tzset "$posix"
            cname=0
          fi
        fi
        ;;
      *"$cachedposix"*)
        iana=$(get_pureiana $iana)
        /opt/ad/bin/tzset "$posix"
        ;;
      "$cachedname"*)
        cname=1
        ;;
      *)
        iana=$(get_pureiana $iana)
        if [ "$posix" != "${posix#<}" ]; then
          posix=${posix#*>}
          local sign=${posix%%[0-9]*}
          [ "$sign" = "-" ] && sign='' || sign='+'
          posix="GMT$sign$posix"
          /opt/ad/bin/tzset "$posix"
        fi
        ;;
    esac
    ;;
  vivotek)
    # cannot write to /etc, so core always sets the TZ env variable
    ;;
  *)
    local tzfile="/usr/share/zoneinfo/$iana"
    if [ -f "$tzfile" ]; then
      parse_tzfile "$tzfile"
      if [ "$posix" != "$desired_posix" ]; then
        tzfile=''
      fi
    else
      tzfile=''
    fi
    if [ -z "$tzfile" ]; then
      tzfile="/usr/share/zoneinfo/Custom"
      write_tzfile "$desired_posix" > "$tzfile"
    fi
    ln -fs "$tzfile" /etc/localtime
    ;;
  esac

  if [ "$os" = "aarch64" ]; then
    if [ 1 -eq $cname ]; then
      echo "$name" > "$namefile.exacq"
    else
      echo "$iana" > "$namefile.exacq"
    fi
  else
    if [ -f "/usr/share/zoneinfo/$iana" ]; then
      echo "$iana" > "$namefile.exacq"
    else
      echo "$name" > "$namefile.exacq"
    fi
  fi
  mv "$namefile.exacq" "$namefile"

  echo "$desired_posix"
}

fix_i610()
{
  # i610 may forget the timezone after reboot
  local posix
  posix=$(paramclient -get -raw system.datetime.tz.name)
  write_tzfile "$posix" > /var/config/localtime.exacq
  ln -fs /var/config/localtime.exacq /var/config/localtime
}

start_ntp()
{
  if ! pidof ntpd > /dev/null && [ -x /etc/init.d/ntp ]; then
    for server in $1; do
      ntpdate -b "$server"
      break
    done
    /etc/init.d/ntp start
  fi
}

get_ntp()
{
  local enabled disabled name value ignore name2 server
  local start="$1"

  while read name value ignore; do
    name2=${name#[#]}
    if [ "$name2" = 'server' -o "$name2" = 'pool' ]; then
      if [ "$value" != 'localhost' -a "$value" = "${value#127.}" ]; then
        if [ "$name" = "$name2" ]; then
          enabled="$enabled $value"
        else
          disabled="$disabled $value"
        fi
      fi
    fi
  done <<EOF
$(sed 's/^#[[:space:]]\+/#/' /etc/ntp.conf)
EOF

  if [ -n "$enabled" ]; then
    echo "enabled"
    for server in $enabled; do
      echo $server
    done
  else
    echo "disabled"
    for server in $disabled; do
      echo $server
    done
  fi

  if [ $start -eq 1 ]; then
    start_ntp "$enabled"
  fi
}

set_ntp()
{
  local enable="$1"
  local servers="$2"
  local driftfile='/var/lib/ntp/ntp.drift'
  local server
  local start="$1"

  /etc/init.d/ntp stop

  if [ -f '/etc/ntp.drift' ]; then
    driftfile='/etc/ntp.drift'
  fi

  {
    for server in $servers; do
      if [ "$enable" = '1' ]; then
        echo "server $server"
      else
        echo "#server $server"
      fi
    done
    cat <<EOF
server 127.127.1.0
fudge 127.127.1.0 stratum 13
driftfile $driftfile
restrict -4 default kod notrap nomodify nopeer noquery
restrict -6 default kod notrap nomodify nopeer noquery
restrict 127.0.0.1
EOF
  } > /etc/ntp.conf.exacq

  mv /etc/ntp.conf.exacq /etc/ntp.conf
  if [ $start -eq 1 ]; then
    if [ "$enable" = '1' ]; then
      start_ntp "$servers"
    else
      start_ntp
    fi
  fi
}

usage()
{
  echo "Usage:"
  echo "$0 [list-timezones|get-timezone]"
  echo "$0 get-ntp [-start-ntp]"
  echo "$0 set-time <timestamp>"
  echo "$0 set-timezone <name|iana>"
  echo "$0 set-ntp [-enable|-disable] [-server <server>]... [-start-ntp]"
  exit 1
}

os='linux'
namefile='/etc/timezone'
zonesfile="${0%/*}/../zones.txt"
##############CACHE VARIABLES START##################
cachedname="_N"
cachediana="_I"
cachedposix="_P"
cachedgmt="_G"

#cache flag for /etc/timezone. If it is set use the name of
#timezone to store if not then use iana from zones
cname=0
#cache flag for /opt/ad/bin/tzset. If it is set query
#/opt/ad/etc/conf/tz-list-uniq.conf with the iana from zones.txt
#and use the posix value from the conf file to set TZ with
#opt/ad/etc/conf/tz-list-uniq.conf
ciana=0
#cache flag for GMT posix values since /opt/ad/bin/tzset
#understands the path "Etc/GMT*"
cgmt=0
#cache flag for /opt/ad/bin/tzset. If it set just use the
#posix value ASIS while setting a timezone with /opt/ad/bin/tzset
#and for matching the value from /etc/localtime with the one
#in zones.txt
cposix=0
##############CACHE VARIABLES END####################

if type parhandclient > /dev/null 2> /dev/null; then
  os='axis'
elif type paramclient > /dev/null 2> /dev/null; then
  os='i610'
  namefile='timezone'
  if [ ! -e "$zonesfile" ]; then
    build_i610
  fi
  if [ ! -e /var/config/localtime -a "$UID" -eq 0 ]; then
    fix_i610
  fi
elif [ -x /opt/ad/bin/tzset ]; then
   build_i825_aarch64
elif [ -e /mnt/auto/CF/vadp ]; then
  os='vivotek'
  namefile='timezone'
fi

case "$1" in
  init)
    ;;
  set-time)
    time=${2%%[^0-9]*}
    if [ -z "$time" ]; then
      usage
    fi
    if [ "$os" = 'linux' ]; then
      # must open /dev/null or date will fail
      date -s "@$time" > /dev/null
      # #15676 - if systemd is installed, it disables hwclock
      # if ntp is also installed, systemd-timesyncd disables itself,
      # and the hardware clock does not get updated if the ntp client
      # does not synchronize to an external server
      hwclock --systohc
    else
      date -s "$time" -D '%s'
    fi
    ;;
  list-timezones)
    list_timezones
    ;;
  get-timezone)
    get_timezone
    ;;
  set-timezone)
    if [ -z "$2" ]; then
      usage
    fi
    set_timezone "$2"
    ;;
  get-ntp)
    shift 1
    start_ntp=0
    if [ -n "$1" ]; then
        if [ "$1" = "-start-ntp" ]; then
            start_ntp=1
        else
            usage
        fi
    fi
    get_ntp "$start_ntp"
    ;;
  set-ntp)
    shift 1
    enable=1
    servers=''
    start_ntp=0
    while [ -n "$1" ]; do
      case "$1" in
        -enable)
          enable=1
          ;;
        -disable)
          enable=0
          ;;
        -server)
          if [ -$z "$2" ]; then
            usage
          fi
          servers="$servers $2"
          shift 1
          ;;
        -start-ntp)
          start_ntp=1
          ;;
        *)
          usage
          ;;
      esac
      shift 1
    done
    if [ "$enable" = '1' -a -z "$servers" ]; then
      usage
    fi
    set_ntp "$enable" "$servers" "$start_ntp"
    ;;
  *)
    usage
    ;;
esac
