#!/bin/sh
#
MMC_DEV_FILE="/dev/mmcblk0"
MNT_POINT="/var/spool/storage/SD_DISK"
INSTALL_DIR=$MNT_POINT/exacq
PKGDIR=/usr/local/packages/ExacqVision

if [ ! -e $MMC_DEV_FILE ]; then
  logger -p user.err -- "ExacqVision -- ********** No CF device"
  exit
fi

if [ ! -e $MNT_POINT ]; then
  logger -p user.err -- "ExacqVision -- ********** CF device not mounted"
  exit
fi

# do this early in the process so we don't need to flush if we have to create the installation files
mount -o remount -o exec $MNT_POINT

need_install()
{
  if [ ! -e $INSTALL_DIR/core ]; then
    return 0
  fi

  # if we don't have md5 info for the install files, we better be safe
  if [ ! -e $PKGDIR/oldmd5.dat ]; then
    logger -t exacqVision -- "Can't find MD5 info."
    return 0
  fi

  # we have old md5 info.  generate new value and compare
  new_md5=$(md5sum $PKGDIR/install/edvrserver.tar.xz | cut -d ' ' -f1)
  old_md5=$(cat $PKGDIR/oldmd5.dat)
  old_md5=${old_md5#*=}

  if [ "$new_md5" != "$old_md5" ]; then
    logger -t exacqVision -- "MD5 has changed." $new_md5 $old_md5
    return 0
  fi

  logger -t exacqVision -- "MD5 has not changed." $new_md5 $old_md5
  return 1
}

install()
{
  if [ -e $INSTALL_DIR ]; then
    cd $INSTALL_DIR
    mkdir -p ../evbackup
    # older update scripts save config xml in the updates directory
    mv edvr logs audit blob eventpi.db pospi.db updates ../evbackup
  fi

  logger -- "ExacqVision -- creating installation files"
  cd $PKGDIR
  rm -rf $INSTALL_DIR
  ./xz -d -c install/edvrserver.tar.xz | tar x -C $MNT_POINT
  logger -- "ExacqVision -- Done expanding tarred installation files"

  cd $INSTALL_DIR
  if [ -e ../evbackup ]; then
    mv ../evbackup/* .
  fi

  mkdir -p edvr logs audit blob downloads data posdata

  if [ -e $PKGDIR/lib/libc.so ]; then
    cp $PKGDIR/lib/libc.so lib
  fi

  cp curl-ca-bundle.crt zones.txt $PKGDIR

  # todo - the following should be deleted after all plugins start using ipcclient
  mkdir -p $PKGDIR/scripts $PKGDIR/lib
  cp scripts/* $PKGDIR/scripts
  cp lib/libcurl* lib/libmbed* $PKGDIR/lib
  cp curl $PKGDIR

#
#  Don't overwrite existing xml
#  Busybox shell cp command doesn't support -n so do it the hard way
  for fil in `ls *.xml`
  do 
    if [ ! -e $PKGDIR/$fil ]
    then
      cp $fil $PKGDIR
    fi
  done
  cd $PKGDIR
  mkdir -p edvr
  cd edvr
  ln -s $INSTALL_DIR/edvr 0
  cd ..
  ln -s $INSTALL_DIR/logs
  ln -s $INSTALL_DIR/audit
  ln -s $INSTALL_DIR/blob
  ln -s $INSTALL_DIR/downloads
  ln -s $INSTALL_DIR/data

# remove or mv renamed copies of event DB
  for fil in `ls eventpi.db.*`
  do
      if [ -L $fil ]
      then
	  rm $fil
      else
	  mv $fil $INSTALL_DIR
      fi
  done
  md5sum $PKGDIR/install/edvrserver.tar.xz | cut -d ' ' -f1 > oldmd5.dat
  logger -- "ExacqVision -- All runtime files created"
}

if need_install; then
  install
fi

cd $PKGDIR/edvr
rm -f 1 2
if [ -e /var/spool/storage/NetworkShare ]; then
    ln -s /var/spool/storage/NetworkShare 1
fi
if [ -e /var/spool/storage/SD_DISK2 ]; then
    ln -s /var/spool/storage/SD_DISK2 2
fi
cd $PKGDIR

# if pos or event database exists on flash, copy to SD & create link
if [ -e pospi.db -a ! -L pospi.db ]
then
mv pospi.db $INSTALL_DIR/posdata
elif [ -e pospi.db ]
then
rm -f pospi.db
fi
ln -s $INSTALL_DIR/posdata/pospi.db pospi.db

if [ -e eventpi.db -a ! -L eventpi.db ]
then
mv eventpi.db $INSTALL_DIR
elif [ -e eventpi.db ]
then
rm -f eventpi.db
fi
ln -s $INSTALL_DIR/eventpi.db eventpi.db

# now do the same for the wal/shm files, but don't create links
if [ -e eventpi.db-shm -a ! -L eventpi.db-shm ]
then
mv eventpi.db-shm $INSTALL_DIR
elif [ -e eventpi.db ]
then
rm -f eventpi.db-shm
fi

if [ -e eventpi.db-wal -a ! -L eventpi.db-wal ]
then
mv eventpi.db-wal $INSTALL_DIR
elif [ -e eventpi.db ]
then
rm -f eventpi.db-wal
fi

if [ -e pospi.db-shm -a ! -L pospi.db-shm ]
then
mv pospi.db-shm $INSTALL_DIR/posdata
elif [ -e pospi.db ]
then
rm -f pospi.db-shm
fi

if [ -e pospi.db-wal -a ! -L pospi.db-wal ]
then
mv pospi.db-wal $INSTALL_DIR/posdata
elif [ -e pospi.db ]
then
rm -f pospi.db-wal
fi

